<?php

namespace MailPoet\Premium\AutomaticEmails\WooCommerce\Events;

use MailPoet\Models\Newsletter;
use MailPoet\Models\ScheduledTask;
use MailPoet\Models\ScheduledTaskSubscriber;
use MailPoet\Models\SendingQueue;
use MailPoet\Models\Subscriber;
use MailPoet\Newsletter\Scheduler\Scheduler;
use MailPoet\Premium\AutomaticEmails\WooCommerce\WooCommerce;
use MailPoet\WP\Hooks;

class FirstPurchase {
  const SLUG = 'woocommerce_first_purchase';
  const ORDER_TOTAL_SHORTCODE = '[woocommerce:order_total]';
  const ORDER_DATE_SHORTCODE = '[woocommerce:order_date]';

  function init() {
    Hooks::addFilter('mailpoet_newsletter_shortcode', array(
      $this,
      'handleOrderTotalShortcode'
    ), 10, 4);
    Hooks::addFilter('mailpoet_newsletter_shortcode', array(
      $this,
      'handleOrderDateShortcode'
    ), 10, 4);

    // We have to use a set of states because an order state after checkout differs for different payment methods
    $accepted_order_states = Hooks::applyFilters('mailpoet_first_purchase_order_states', ['completed', 'processing']);

    foreach($accepted_order_states as $state) {
      Hooks::addAction('woocommerce_order_status_' . $state, [
        $this,
        'scheduleEmailWhenOrderIsPlaced'
      ], 10, 1);
    }
  }

  function getEventDetails() {
    return array(
      'slug' => self::SLUG,
      'title' => __('First Purchase', 'mailpoet-premium'),
      'description' => __('Let MailPoet send an email to customers who make their first purchase.', 'mailpoet-premium'),
      'listingScheduleDisplayText' => __('when first purchase is made', 'mailpoet-premium'),
      'badge' => array(
        'text' => __('Must-have', 'mailpoet-premium'),
        'style' => 'red'
      ),
      'shortcodes' => array(
        array(
          'text' => __('Order amount', 'mailpoet-premium'),
          'shortcode' => self::ORDER_TOTAL_SHORTCODE
        ),
        array(
          'text' => __('Order date', 'mailpoet-premium'),
          'shortcode' => self::ORDER_DATE_SHORTCODE
        )
      )
    );
  }

  function handleOrderDateShortcode($shortcode, $newsletter, $subscriber, $queue) {
    if($shortcode !== self::ORDER_DATE_SHORTCODE) return $shortcode;

    $default_value = date_i18n(get_option('date_format'));
    if(!$queue) return $default_value;

    $meta = $queue->getMeta();
    return (!empty($meta['order_date'])) ? date_i18n(get_option('date_format'), $meta['order_date']) : $default_value;
  }

  function handleOrderTotalShortcode($shortcode, $newsletter, $subscriber, $queue) {
    if($shortcode !== self::ORDER_TOTAL_SHORTCODE) return $shortcode;

    $default_value = wc_price(0);
    if(!$queue) return $default_value;

    $meta = $queue->getMeta();
    return (!empty($meta['order_amount'])) ? wc_price($meta['order_amount']) : $default_value;
  }

  function scheduleEmailWhenOrderIsPlaced($order_id) {
    $order_details = wc_get_order($order_id);
    if(!$order_details || !$order_details->get_customer_id()) return;

    $customer_order_count = wc_get_customer_order_count($order_details->get_customer_id());
    if($customer_order_count > 1) return;

    $meta = array(
      'order_amount' => $order_details->total,
      'order_date' => $order_details->get_date_created()->getTimestamp(),
      'order_id' => $order_details->get_id()
    );

    $subscriber = Subscriber::where('wp_user_id', $order_details->get_customer_id())->findOne();
    if(!$subscriber) return;

    $check_email_was_not_scheduled = function (Newsletter $newsletter) use ($subscriber) {
      $count = (int)SendingQueue::rawQuery(
        "SELECT COUNT(*) as count
        FROM `" . SendingQueue::$_table . "`
        JOIN `" . ScheduledTask::$_table . "` ON " . SendingQueue::$_table . ".task_id = " . ScheduledTask::$_table . ".id
        JOIN `" . ScheduledTaskSubscriber::$_table . "` ON " . ScheduledTask::$_table . ".id = " . ScheduledTaskSubscriber::$_table . ".task_id
        WHERE " . ScheduledTaskSubscriber::$_table . ".subscriber_id = " . $subscriber->id . "
        AND " . SendingQueue::$_table . ".newsletter_id = " . $newsletter->id
      )->findOne()->count;
      return !$count;
    };

    Scheduler::scheduleAutomaticEmail(WooCommerce::SLUG, self::SLUG, $check_email_was_not_scheduled, $subscriber->id, $meta);
  }
}
